/*
 *  @file   TilerMemoryMgr.c
 *
 *  @brief   Implements  wrapper functions over Heap  APIs and Shared region
 *           address translation API. Provides API to work on shared memory
 *           over which heap and shared region is created. instance  create takes
 *           heap handle created on shared memory for allocations.
 *
 *
 *  ============================================================================
 *
 *  Copyright (c) 2008-2009, Texas Instruments Incorporated
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  Contact information for paper mail:
 *  Texas Instruments
 *  Post Office Box 655303
 *  Dallas, Texas 75265
 *  Contact information: 
 *  http://www-k.ext.ti.com/sc/technical-support/product-information-centers.htm?
 *  DCMP=TIHomeTracking&HQS=Other+OT+home_d_contact
 *  ============================================================================
 *  
 */



/*-------------------------    XDC  specific includes ----------------------  */
#include <xdc/std.h>
#include <xdc/runtime/Memory.h>
#include <xdc/runtime/System.h>
#include <xdc/runtime/Startup.h>
#include <xdc/runtime/Assert.h>
#include <xdc/runtime/Error.h>
#include <xdc/runtime/IHeap.h>

/*-------------------------  BIOS includes ---------------------------------- */


/*-------------------------    Generic includes  ---------------------------- */
#include <string.h>

/*-------------------------    IPC module specific includes ------------------*/
#include <ti/sdo/ipc/_SharedRegion.h>

#include <ti/syslink/inc/SyslinkMemMgr_errBase.h>

#include "package/internal/TilerMemoryMgr.xdc.h"

#if defined (SYSLINK_USE_TILER)
#include <ti/sdo/tiler/MemMgr.h>


#define MEMSET(addr, val, count)        \
{                                       \
    UInt i;                             \
    UInt8 *ptr = (UInt8 *)(addr);       \
                                        \
    for (i = 0; i < count; i++) {       \
        *ptr++ = val;                   \
    }                                   \
}

#endif /* #if defined (SYSLINK_USE_TILER) */


#if defined (__cplusplus)
extern "C" {
#endif



/** ============================================================================
 *  External APIs
 *  ============================================================================
 */

/*
 *  ======== SyslinkMemMgr_TilerMemory_Params_init ========
 *  Function to get the default parameters of the TilerMemory SyslinkMemMgr.
 *
 *  This function can be called by the application to get their
 *  default parameters.
 *
 */
// void SyslinkMemMgr_TilerMemory_Params_init(SyslinkMemMgr_TilerMemory_Params *params)
// {
//     if(params != NULL) {
//         params->heapHandle = NULL;
//     }
// }


Int TilerMemoryMgr_Instance_init(TilerMemoryMgr_Object         *obj,
                                 const TilerMemoryMgr_Params  *params,
                                 Error_Block                  *eb)
{

    Int32 status = MEMMGR_SUCCESS;

    //Assert_isTrue(obj != NULL, TilerMemoryMgr_A_nullArgument);
    obj->heapHandle = params->heapHandle;

    return(status);
}

Void TilerMemoryMgr_Instance_finalize(TilerMemoryMgr_Object *obj,
                                     Int                   status)
{
    //Assert_isTrue(obj != NULL, TilerMemoryMgr_A_nullArgument);
    obj->heapHandle = NULL;

}

/* Function to allocate memory from the SyslinkMemMgr */
Ptr
TilerMemoryMgr_alloc (TilerMemoryMgr_Object *obj,
                      Ptr                    allocParams)
{
    Ptr   buf = NULL;
#if defined (SYSLINK_USE_TILER)
    TilerMemoryMgr_allocParams  * instAllocParams =
                                      (TilerMemoryMgr_allocParams  *)allocParams;

    /* TBD: This function should be able to allocate multiple blocks*/
    MemAllocBlock memAllocBlk;

    /* Memset is neccessary before MemMgr_Alloc RCM server raises assertions otherwise*/
    MEMSET(&memAllocBlk, 0, sizeof(memAllocBlk));

    memAllocBlk.pixelFormat     = instAllocParams->block[0].pixelFormat;
    memAllocBlk.dim.area.width  = instAllocParams->block[0].dim.area.width;
    memAllocBlk.dim.area.height = instAllocParams->block[0].dim.area.height;

    if (obj != NULL) {
        buf = MemMgr_Alloc(&memAllocBlk,
                           instAllocParams->tilerBlocks);
        if(buf == NULL){
            System_printf("\nTiler MemMgr_Alloc failed\n");
        }
        else {
            instAllocParams->block[0].stride = memAllocBlk.stride;
        }
    }
#endif /* #if defined (SYSLINK_USE_TILER) */
    return (buf);
}

/* Function to allocate memory from the SyslinkMemMgr */
Int32
TilerMemoryMgr_free (TilerMemoryMgr_Object *obj,
                     Ptr                    freeParams)
{
    Int32 status = MEMMGR_SUCCESS;
#if defined (SYSLINK_USE_TILER)
    TilerMemoryMgr_freeParams  * instFreeParams =
                                      (TilerMemoryMgr_freeParams  *)freeParams;
    if (obj == NULL) {
        status = MEMMGR_E_FAIL;
    }
    else {
        MemMgr_Free(instFreeParams->ptr);
    }
#endif /* #if defined (SYSLINK_USE_TILER) */
    return (status);
}

/* Function to allocate memory from the SyslinkMemMgr.Calls sharedregion API to do
 * address translations
 */
Ptr
TilerMemoryMgr_translate (TilerMemoryMgr_Object      *obj,
                          Ptr                         srcAddr,
                          TilerMemoryMgr_AddrType     srcAddrType,
                          TilerMemoryMgr_AddrType     desAddrType)
{
    Ptr       dstAddr = NULL;

    /* Currently translation APIs are not provided by MemMgr so return the same pointer*/

    dstAddr = srcAddr;

    return (dstAddr);

}


#if defined (__cplusplus)
}
#endif /* defined (__cplusplus) */






